@@ -1,11 +1,11 @@
 package SNMP;
-$VERSION = 1.6;   # current release version number
+$VERSION = 1.7;   # current release version number
 
 require Exporter;
 require DynaLoader;
 require AutoLoader;
 
-@SNMP::ISA = qw(Exporter Autoloader DynaLoader);
+@SNMP::ISA = qw(Exporter AutoLoader DynaLoader);
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
@@ -57,11 +57,17 @@ bootstrap SNMP;
 
 # Preloaded methods go here.
 $auto_init_mib = 1; # set to true, mib is loaded on session creation
-                    # set to zero(0) for manual control
+                    # set to zero(0) to disable auto-load of mib -
+                    # user should call setMib to load mib
+$verbose = 0; # set to false, limit extraneous I/O
+
 
 sub setMib {
-# re-initializes mib with file name provided
-   my ($file,$force) = @_;
+# loads mib from file name provided
+# won't override currently loaded mib unless second arg(force) is non-zero
+   my $file = shift;
+   my $force = shift || '0';
+   return 0 if $file and not (-r $file);
    SNMP::_setmib($file,$force);
 }
 
@@ -79,12 +85,26 @@ sub translateObj {
    }
 }
 
+sub getType {
+# return SNMP data type for given textual identifier
+# OBJECTID, OCTETSTR, INTEGER, NETADDR, IPADDR, COUNTER
+# GAUGE, TIMETICKS, OPAQUE, or undef
+  my $tag = shift;
+  SNMP::_get_type($tag);
+}
+
+sub mapEnum {
+  my $varbind = shift;
+
+  SNMP::_map_enum($varbind->[$SNMP::Varbind::tag_f]);
+}
+
 package SNMP::Session;
 
 sub new {
    my $type = shift;
    my $this = {};
-   my ($name, $aliases, $type, $len, $thisaddr);
+   my ($name, $aliases, $host_type, $len, $thisaddr);
 
    %$this = @_;
 
@@ -96,8 +116,14 @@ sub new {
    # v1 or v2, defaults to v1
    $this->{Version} ||= 1;
 
-   # community defaults to public 
-   $this->{Community} ||= 'public'; 
+   # allow override of remote SNMP port
+   $this->{RemotePort} ||= 161;
+
+   # destination host defaults to localhost
+   $this->{DestHost} ||= 'localhost';
+
+   # community defaults to public
+   $this->{Community} ||= 'public';
 
    # number of retries before giving up, defaults to SNMP_DEFAULT_RETRIES
    $this->{Retries} = SNMP::SNMP_DEFAULT_RETRIES() unless defined($this->{Retries});
@@ -105,21 +131,20 @@ sub new {
    # timeout before retry, defaults to SNMP_DEFAULT_TIMEOUT
    $this->{Timeout} = SNMP::SNMP_DEFAULT_TIMEOUT() unless defined($this->{Timeout});
 
-   # convert to dotted ip addr if needed 
-#   if ($this->{DestHost} =~ /[0-255]\.[0-255]\.[0-255]\.[0-255]/) {
+   # convert to dotted ip addr if needed
    if ($this->{DestHost} =~ /\d+\.\d+\.\d+\.\d+/) {
-      $this->{DestAddr} = $this->{DestHost};
-   } elsif (defined($this->{DestHost})) {
-     ($name, $aliases, $type, $len, $thisaddr) = 
-        gethostbyname($this->{DestHost});
-      $this->{DestAddr} = join('.', unpack("C4", $thisaddr));
+     $this->{DestAddr} = $this->{DestHost};
    } else {
-     warn("undefined destination host!");
+     ($name, $aliases, $host_type, $len, $thisaddr) =
+       gethostbyname($this->{DestHost});
+     $this->{DestAddr} = join('.', unpack("C4", $thisaddr));
    }
+   warn("undefined destination address!") unless $this->{DestAddr};
 
    $this->{SessPtr} = SNMP::_new_session($this->{Version},
 					 $this->{Community},
 					 $this->{DestAddr},
+					 $this->{RemotePort},
 					 $this->{Retries},
 					 $this->{Timeout},
 					);
@@ -147,7 +172,7 @@ sub set {
      my $val = shift;
      $varbind_list_ref = [[$tag, $iid, $val]];
    }
-  
+
    $res = SNMP::_set($this, $varbind_list_ref);
 
 }
@@ -169,12 +194,43 @@ sub get {
      my ($tag, $iid) = ($vars =~ /^(\w+)\.(.*)$/);
      $varbind_list_ref = [[$tag, $iid]];
    }
-  
+
    @res = SNMP::_get($this, $this->{RetryNoSuch}, $varbind_list_ref);
 
    return(wantarray() ? @res : $res[0]);
 }
 
+sub fget {
+   my $this = shift;
+   my $vars = shift;
+   my ($varbind_list_ref, @res);
+
+
+   if (ref($vars) =~ /SNMP::VarList/) {
+     $varbind_list_ref = $vars;
+   } elsif (ref($vars) =~ /SNMP::Varbind/) {
+     $varbind_list_ref = [$vars];
+   } elsif (ref($vars) =~ /ARRAY/) {
+     $varbind_list_ref = [$vars];
+     $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
+   } else {
+     my ($tag, $iid) = ($vars =~ /^(\w+)\.(.*)$/);
+     $varbind_list_ref = [[$tag, $iid]];
+   }
+
+   @res = SNMP::_get($this, $this->{RetryNoSuch}, $varbind_list_ref);
+
+   foreach $varbind (@$varbind_list_ref) {
+     if ($sub = $this->{VarFormats}{$varbind->[$SNMP::Varbind::tag_f]}) {
+       $varbind->[$SNMP::Varbind::val_f] = &$sub($varbind);
+     } elsif ($sub = $this->{TypeFormats}{$varbind->[$SNMP::Varbind::type_f]}){
+       $varbind->[$SNMP::Varbind::val_f] = &$sub($varbind);
+     }
+   }
+
+   return(wantarray() ? @res : $res[0]);
+}
+
 sub getnext {
    my $this = shift;
    my $vars = shift;
@@ -191,17 +247,19 @@ sub getnext {
      my ($tag, $iid) = ($vars =~ /^(\w+)\.(.*)$/);
      $varbind_list_ref = [[$tag, $iid]];
    }
-  
+
    @res = SNMP::_getnext($this, $varbind_list_ref);
 
    return(wantarray() ? @res : $res[0]);
 }
 
+
 package SNMP::Varbind;
 
 $tag_f = 0;
 $iid_f = 1;
 $val_f = 2;
+$type_f = 3;
 
 sub new {
    my $type = shift;
@@ -210,6 +268,22 @@ sub new {
    bless $this;
 }
 
+sub tag {
+  $_[0]->[$tag_f];
+}
+
+sub iid {
+  $_[0]->[$iid_f];
+}
+
+sub val {
+  $_[0]->[$val_f];
+}
+
+sub type {
+  $_[0]->[$type_f];
+}
+
 package SNMP::VarList;
 
 sub new {
